/*
 * disks.c - disk selection fuctions.
 *           these are an extension of the dtaz.c code.
 *
 * $Id: disks.c 25 2006-02-09 00:45:06Z brendan $
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * COPYRIGHT: Copyright (c) 2006 Brendan Gregg.
 *
 * 29-Jan-2006  Brendan Gregg   Created this
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <strings.h>
#include <dirent.h>

#include "dtaz.h"
#include "disks.h"
#include "dialog.h"
#include "statusbar.h"

/* device to trace, global */
dev_t		disks_edev;

/* set dt_plot_max_raw to a default value */
static void	disks_set_default_max_block(GtkWidget *);

/* set dt_plot_max_raw to the size of a given disk path */
static void	disks_set_path_max_block(gchar *);

int
disks_set_edev(GtkWidget *selector)
{
	gchar		*path;
	struct stat	filestat;

	/* fetch selected value */
	path = g_strdup(
	    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(selector)->entry)));

	if (strcmp(path, "<choose disk>") == 0) {
		sb_err(dt_status, "You need to choose a disk.");
		return (FALSE);
	}

	if (stat(path, &filestat) != 0) {
		sb_err(dt_status, "couldn't stat selected disk.");
		return (FALSE);
	}

	/* set edev */
	disks_edev = filestat.st_rdev;

	g_free(path);

	return (TRUE);
}

void
disks_find(GtkWidget *widget, const char *dir)
{
	DIR		*dirp;
	struct dirent	*dp;
	GList 		*glist = NULL;
	char		buf[LINE_LEN];
	char		*path;

	/* glist contains combo box text */
	glist = g_list_append(glist, "<choose disk>");

	/* find disk devices */
	if ((dirp = opendir(dir)) == NULL) {
		fprintf(stderr, "couldn't read %s", dir);
		perror(": ");
		exit(1);
	}
	do {
		if ((dp = readdir(dirp)) != NULL) {

			if (strcmp(dp->d_name, ".") == 0) continue;
			if (strcmp(dp->d_name, "..") == 0) continue;

			/*
			 * make full path. we delibratly store the full
			 * path in the combo box to make it clear which
			 * devices we are examining.
			 */
			sprintf(buf, "%s/%s", dir, dp->d_name);
			path = g_strdup(buf);
			/* no memory leeks to see here, move along.. */

			/* add disk entry to glist */
			glist = g_list_append(glist, path);
		}
	} while (dp != NULL);

	/* add glist to combo box */
	gtk_combo_set_popdown_strings(GTK_COMBO(widget), glist);

	closedir(dirp);
}

void
disks_combo_select(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gchar *choosen;

	/* fetch selected value */
	choosen = g_strdup(
	    gtk_entry_get_text(GTK_ENTRY(widget)));

	/* change statusbar message */
	if (strcmp(choosen, "<choose disk>") == 0)
		sb_msg(dt_status, "Select a DISK.");
	else {
		/* clear string, ready to trace */
		sb_msg(dt_status, "");

		/* set max on plot */
		disks_set_path_max_block(choosen);
	}

	g_free(choosen);
}

void
disks_set_default_max_block(GtkWidget *window)
{

	/* create a popup to explain the prtvtoc error */
	dialog_message(window, "WARNING", dialog_prtvtoc_msg);

	/* update statusbar */
	sb_msg(dt_status, "WARNING: prtvtoc failed, block max = default");

	/*
	 * we shouldn't let dt_plot_max_raw be 0, it's used in
	 * many divisions later on - which could each be checked,
	 * adding overhead, so we ensure it doesn't happen here.
	 */
	dt_plot_max_raw = DT_PLOT_MAX_DEF;

	/* set scale mode */
	dt_scale = DT_SCALE_DISCOVERY;
}

void
disks_set_path_max_block(gchar *path)
{
	gchar	cmd[LINE_LEN];
	gchar	buf[LINE_LEN];
	int	block_max;
	int	slice;
	uint	part;
	uint	tag;
	uint	flag;
	long	start = 0;
	long	size;
	gchar	*ptr;
	FILE	*fin;

	/* the combo box sometimes does this */
	if (strcmp(path, "") == 0)
		return;

	/* find slice from /dev/dsk pathname */
	if ((ptr = strrchr(path, 's')) == NULL) {
		sb_err(dt_status, "couldn't determine slice");
		return;
	}

	/* got slice num */
	slice = atoi(++ptr);

	/* consrtuct prtvtoc cmd */
	sprintf(cmd, "/usr/sbin/prtvtoc -h %s", path);

	/* run prtvtoc */
	if ((fin = popen(cmd, "r")) == NULL) {

		/* set discovery mode and popup a dialog to explain */
		disks_set_default_max_block(GTK_WINDOW_TOPLEVEL);
		return;
	}

	/* process prtvtoc output */
	block_max = 0;
	while (fgets(buf, LINE_LEN, fin) != NULL) {

		/* fetch size */
		sscanf(buf, "      %2u  %5u    %02x  %9lu %9lu",
		    &part, &tag, &flag, &start, &size);

		/* match on slice */
		if (part == slice) {
			block_max = size;
			break;
		}
	}

	pclose(fin);
	/* not catching this on purpose */

	/* check for slice 2 at start of disk */
	if (slice == 2 && start == 0)
		dialog_message(GTK_WINDOW_TOPLEVEL, "WARNING",
		    dialog_slice_2_msg);

	if (block_max <= 0)
		/* set discovery mode and popup a dialog to explain */
		disks_set_default_max_block(GTK_WINDOW_TOPLEVEL);
	else {
		/* set globals */
		dt_scale = DT_SCALE_KNOWN;
		dt_plot_max_raw = block_max;
	}

	/* set new label */
	sprintf(buf, "Block\n%llu", (u_longlong_t)dt_plot_max_raw);
	gtk_label_set_text(GTK_LABEL(dt_plot_label[2]), buf);
}
