/*
 * plot.c - functions for plotting disk event data.
 *          these are an extension of the dtaz.c code.
 *
 * $Id: plot.c 31 2006-02-09 15:48:09Z brendan $
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * COPYRIGHT: Copyright (c) 2006 Brendan Gregg.
 *
 * 29-Jan-2006  Brendan Gregg   Created this
 */

#include <gtk/gtk.h>

#include "dtaz.h"
#include "colours.h"
#include "dtrace.h"
#include "plot.h"

int
plot_get_index(long long block)
{
	uint		index;
	char		buf[LINE_LEN];

	/* check for scale discovery */
	if (dt_scale == DT_SCALE_DISCOVERY && block > dt_plot_max_raw) {

		dt_plot_max_raw = block * 1.05; 	/* headroom */

		/* set new label */
		sprintf(buf, "Block\n%llu", (u_longlong_t)dt_plot_max_raw);
		gtk_label_set_text(GTK_LABEL(dt_plot_label[2]), buf);

		/* this may have happened */
		dt_plot_resized = TRUE;
	}

	/* save calculation */
	index = (block * DT_PLOT_MAX_Y) / dt_plot_max_raw;

	/* check limits */
	if (index > DT_PLOT_MAX_Y) {
		g_print("WARNING: index == %d\n", index);
		index = DT_PLOT_MAX_Y;
	}
	if (index < 0) {
		g_print("WARNING: index == %d\n", index);
		index = 0;
	}

	return (index);
}

int
plot_get_bucket(int index, int max_y)
{
	uint		bucket;
	static double	ratio;

	/*
	 * change ratio
	 *
	 * this function in called often, and so to reduce the overhead
	 * we cache much of the calculation as the variable "ratio".
	 *
	 * the final algorithm takes a index address "index", the maximum
	 * index address possible "DT_PLOT_MAX_Y", and determines the bucket
	 * value for plotting. It takes into acconut the size of the pixels
	 * "dt_resolution", and the maximum plot size in pixels "max_y".
	 */
	if (dt_plot_resized == TRUE) {
		ratio = (max_y - dt_resolution) / dt_resolution;
		ratio = ratio / DT_PLOT_MAX_Y;
		dt_plot_resized = FALSE;
	}

	/* do calculation */
	bucket = ratio * index;

	/* check limits */
	if (bucket > (max_y / dt_resolution)) {
		g_print("WARNING: bucket == %d\n", bucket);
		bucket = (max_y / dt_resolution);
	}
	if (bucket < 0) {
		g_print("WARNING: bucket == %d\n", bucket);
		bucket = 0;
	}

	return (bucket);
}

uint
plot_get_seek_loc(long long seek, int max_y)
{
	uint		loc;

	/* scale discovery is done in get_plot_bucket function */

	/*
	 * this takes a seek distance  "seek", the maximum seek distance
	 * possible (which turns out to be the size of the entire slice,
	 * "dt_plot_max_raw"), and determines the location value
	 * for plotting. It takes into acconut the maximum plot size in
	 * pixels "max_y".
	 */
	loc = max_y - 1 - ((max_y - 1) * seek) / dt_plot_max_raw;

	/* check limits */
	if (loc > max_y) {
		g_print("WARNING: seek loc == %d(%d)\n", loc, max_y);
		loc = max_y;
	}
	if (loc < 0) {
		g_print("WARNING: seek loc == %d(%d)\n", loc, 0);
		loc = 0;
	}

	return (loc);
}

void
plot_new_screen(void)
{
	int	i;
	int	j;
	char	buf[LINE_LEN];

	/* stop tracing if needed */
	if (dt_state == DT_STATE_STARTED) {
		stop_tracing();
	}

	/* clear canvas data */
	for (j = 0; j < DT_PLOT_MAX_X; j++)
		for (i = 0; i < DT_PLOT_MAX_Y; i++)
			dt_plot_canvas[j][i] = 0;
	for (i = 0; i < DT_PLOT_MAX_X; i++) {
		dt_seek_canvas[i].avg = 0;
		dt_seek_canvas[i].max = 0;
	}

	/* clear plots */
	dt_plot_exposed = TRUE;

	/* set block max */
	if (dt_scale == DT_SCALE_DISCOVERY) {
		dt_plot_max_raw = DT_PLOT_MAX_DEF;
		sprintf(buf, "Block\n%llu", (u_longlong_t)dt_plot_max_raw);
		gtk_label_set_text(GTK_LABEL(dt_plot_label[2]), buf);
	}

	/* reset globals */
	dt_current_x = 0;
	dt_time = 0;
}
